

mps.pgmload <- function(fname) {
  f <- file(fname,open="rb")
  header <- readLines(f,4,FALSE)
  if( header[[1]] != "P5" ) 
    stop("File appears to be no PGM")
  bh <- as.numeric(unlist(strsplit(header[[3]]," ")))
  if(length(bh)!=2 )
    stop("File appears to be not a plain image")
  X <- readBin(f,"integer",prod(bh),1,signed=FALSE)
  close(f)
  mps.grayimage(X,bh)
}

mps.pgmwrite <- function(X,fname,values=c(128,0,255),bw=FALSE) {
  f <- file(fname,open="wb")
  header <- c("P5","# Created by the mps R-package",paste(dim(X),collapse=" "),"255")
  writeLines(header,f)
  if( bw )
    writeBin(values[X+1],f,1)
  else 
    writeBin(c(unclass(X)),f,1)
  close(f)
}


mps.cutoff <-function(X,cutoff=0.1 ) {
  mps.image(ifelse(X<cutoff,0,1),dim(X))
}

mps.image <- function(x,dim,scale=1,...) {
  structure(as.integer(x),class="mpsimage",dim=dim,scale=scale,...)
}

mps.grayimage <- function(x,dim,scale=1,...) {
  structure(as.integer(x),class="mpsgrayimage",dim=dim,scale=scale,...)
}

mps.floatimage <- function(x,dim,scale=1,...) {
  structure(as.numeric(x),class="mpsfloatimage",dim=dim,scale=scale,...)
}


plot.mpsimage <- function(x,...,col=NULL,nacol="gray",m=max(x)+1,v=unique(x)) {
  v <- sort(unique(c(-1,v)))
  if( is.null(col) ) {
    if( all( v %in% c(-1,0,1)) )
      col <- c(nacol,"black","white")
    else if( all(v[v!=-1] >=0) ) {
      col <- c(nacol,rainbow(length(v[v!=-1])))
    } else {
      v <- sort(unique(c(0,v,-v)))
      n <- sum(v>0)
      col <- c(rev(rainbow(n,0.5)),"black",rainbow(n,1))
    }
  } else  col <- col[1:length(v)]
  print(v)
  bk <- (c(min(v)-1,v)+c(v,max(v)+1))/2
  print(col)
  print(bk)
  Y <- 1:nrow(x)
  X <- 1:ncol(x)
  eqscplot(X,Y,type="n",xlim=c(0,ncol(x)),ylim=c(nrow(x),0))
  x[x<0 | x>=m]<- -1
  image(X,Y,x,breaks=bk,col=col,add=T)
}

plot.mpsgrayimage <- function(x,...,col=NULL,m=255) {
  Y <- 1:nrow(x)
  X <- 1:ncol(x)
  eqscplot(X,Y,type="n",xlim=c(0,ncol(x)),ylim=c(nrow(x),0))
  image(X,Y,x,breaks=(0:256)-0.5,col=hsv(0,0,seq(0,1,length.out=256)),add=T)
}

plot.mpsfloatimage <- function(x,...,m=max(x),zlim=c(0,1)) {
  plot(mps.grayimage(as.integer((x-zlim[1])/(zlim[2]-zlim[1])*255),dim(x)))
}

mps.tabulate <- function(X,locs) {
  if( length(dim(locs)) != 2 )
    stop("locs must be a matrix")
  maxValue <- as.integer(max(X)+1)
  erg <- .C(mpsCtabulate,
            maxValue=maxValue,
            ndim    =as.integer(length(dim(X))),
            dim     =as.integer(dim(X)),
            image   =as.integer(X),
            minI    =as.integer( -apply(locs,2,min)+1 ),
            maxI    =as.integer( dim(X)-apply(locs,2,max)-1 ),
            difdim  =as.integer( dim(locs) ),
            dif     =as.integer( locs ),
            erg     =integer(maxValue^nrow(locs))
            )$erg
  dim(erg) <- rep(maxValue,nrow(locs))
  invisible(as.table(erg))
}

mps.wimage <- function(X,weights,dist) {
  maxValue <- as.integer(length(weights))
  mps.floatimage(exp(.C(mpsCwimage,
                    maxValue=maxValue,
                    ndim    =as.integer(length(dim(X))),
                    dim     =as.integer(dim(X)),
                    image   =as.integer(X),
                    weights =as.numeric(log(weights)),
                    dist    =as.numeric(dist),
                    wimage  =numeric(prod(dim(X)))
                    )$wimage),dim(X))
}

mps.weightedtabulate <- function(X,locs,weights,dist=30) {
  if( length(dim(locs)) != 2 )
    stop("locs must be a matrix")
  maxValue <- as.integer(length(weights))
  wimage <- mps.wimage(X,weights,dist=dist)
  erg <- .C(mpsCweightedtabulate,
            maxValue=maxValue,
            ndim    =as.integer(length(dim(X))),
            dim     =as.integer(dim(X)),
            image   =as.integer(X),
            minI    =as.integer( -apply(locs,2,min)+1 ),
            maxI    =as.integer( dim(X)-apply(locs,2,max)-1 ),
            difdim  =as.integer( dim(locs) ),
            dif     =as.integer( locs ),
            wimage  =as.numeric(wimage),
            erg     =numeric(maxValue^nrow(locs))
            )$erg
  dim(erg) <- rep(maxValue,nrow(locs))
  invisible(as.table(erg))
}



xxapply <- function(l,fun,...) {
  erg <- lapply(l,fun,...)
  dims <- lapply(erg,dim)
  dim1 <- dims[[1]]
  if( ! all(unlist(lapply(dims,function(x) length(dim1)==length(x)&&all(dim1==x))))) {
    print(dims)
    stop("xxapply dim error")
  }
  x <- unlist(erg)
  dim(x) <- c(prod(dim1),length(erg))
  x <- x %*% rep(1,length(erg))
  dim(x)<- dim1
  x
}

mps.xtabulate <- function(X,locs,scale=1,rot=0,outer=FALSE,rescale=1,sym=FALSE,weights=NULL) {
  if( outer ) {
    nscale <- length(scale)
    nrot   <- length(rot)
    scale <- rep(scale,nrot)
    rot   <- rep(rot,each=nscale)
  }
  sc    <- scale*rescale
  if( length(sc)!=length(rot) )
    stop("scale and rot of different length")
  erg <- xxapply(1:length(rot),
                 function(i,img,sc) {
                   w <- rot[i]*pi/180
                   nloc <- (locs * sc[i]) %*%
                                      matrix(c(cos(w),sin(w),-sin(w),cos(w)),
                                             nrow=2)
                   if( is.null(weights) )
                     mps.tabulate(img,nloc)
                   else
                     mps.weightedtabulate(img,nloc,weights)
                 },img=X,sc=sc)
  if( !identical(sym,FALSE) ) {
    if( length(sym)== 1 && sym )
      sym <- attr(locs,"sym")
    erg <- xxapply(1:ncol(sym),function(i) aperm(erg,sym[,i])  )
  }
  invisible(as.table(erg))
}




mps.reweight <- function(tab,weights) {
  if( is.null(weights) )
    return(tab)
  dimT <- dim(tab)
  MaxV <- dimT[1]
  ldim <- length(dimT)
  osum <- sum(tab)
  if( length(weights) != MaxV )
    stop("weights are not matching")
  if( any(dimT!=MaxV) )
    stop("tab is not a autotabulation")
  tab <- .C(mpsCreweight,
            ldim=as.integer(ldim),
            dim=as.integer(dimT),
            tab=as.numeric(tab),
            weights=as.numeric(weights)
            )$tab
  tab <- tab/sum(tab)*osum
  return(as.table( structure(tab,dim=dimT) ))
}


mps.changespecificy <- function(tab,specificy=0,indep=0,min=0,
                                mar=apply(tab,1,sum)) {
  osum <- sum(tab)
  tab <- tab/sum(tab)
  mar <- mar/sum(mar)
  R<-mar
  for( i in 2:length(dim(tab)))
    R <- R %o% mar
  tab <-(exp(log(((1-indep)*tab/R+indep+min))*exp(specificy))-min)*R
  as.table(tab/sum(tab)*osum)
}

mps.entropy <- function(tab,ref=1) {
  elim <-function(x) structure(ifelse(is.nan(x),0,x),dim=dim(x))
  if( is.table(tab) ) {
    if( ! is.table(ref) && length(ref)==dim(tab)[1] ) {
      oref<-ref
      for( i in 2:length(dim(tab)))
        ref <- ref%o%oref
    }
    tab <- tab/sum(tab)
    -sum(elim(log(tab/ref,2)*tab))
  } else if(is.matrix(tab)) {
    s <- c(tab %*% rep(1,ncol(tab)))
    tab <-tab/s
    -c((elim(log(tab/ref,2)*tab))%*% rep(1,ncol(tab)))
  } else {
    tab <- tab/sum(tab)
    -sum(elim(log(tab/ref,2)*tab))
  }
}



mps.independent <- function(tab,mar=apply(tab,1,sum)) {
  osum <- sum(tab)
  tab <- tab/sum(tab)
  mar <- mar/sum(mar)
  R<-mar
  for( i in 2:length(dim(tab)))
    R <- R %o% mar
  R/sum(R)*osum
}
  
mps.locSetStar <- function(dist,nangle,phi=0) {
  w <- (phi+seq(0,360,length.out=nangle+1)[-(nangle+1)])*pi/180
  syms <- rbind(1,1+sapply(1:nangle,
                           function(i)  {
                             rep(1+((-1:(nangle-2))+i)%%nangle,
                                 length(dist))
                           }
                           ))
  syms <- cbind(syms,syms[c(1,nrow(syms):2),])
  structure(t(matrix(c(0,0,unlist(lapply(dist,
                                          function(d) {
                                            rbind(d*sin(w),d*cos(w))
                                          }
                                          )
                                  )),nrow=2)),sym=syms)
}


mps.sparseInterp <- function(timage,iimage,OutLoc,InLocs,
                             MaxValue=max(c(timage))+1,
                             minTotal=15,
                             acceptWrongs=0,
                             weights=rep(1,MaxValue)) {
  ndim         <- length(dim(timage));
  OutLoc       <- t(OutLoc)
  InLocs       <- lapply(InLocs,t)
  dimsInLocs   <- sapply(InLocs,dim)
  if( is.null(weights) )
    weights<-rep(1,MaxValue)
  if( length(dim(iimage)) !=ndim )
    stop("mps.sparseInterp: length(dim(iimage)) !=ndim ")
  if( dim(OutLoc)[1]!=ndim )
    stop("mps.sparseInterp: dim(OutLoc)[2]!=ndim ")
  if( dim(OutLoc)[1]!=ndim )
    stop("mps.sparseInterp: dim(OutLoc)[2]!=ndim")
  if( nrow(dimsInLocs)!= 2 )
    stop("mps.sparseInterp: nrow(dimsInLocs)!= 2")
  if( any(dimsInLocs[1,]!=ndim) )
    stop("mps.sparseInterp: any(dimsInLocs[1,]!=ndim)")
  if( ncol(OutLoc)!=length(InLocs) )
    stop("mps.sparseInterp: length(OutLoc)!=length(InLocs)")
  if( length(dimsInLocs)!=2*ncol(OutLoc) )
    stop("mps.sparseInterp: length(dimsInLocs)!=2*length(OutLoc) ")
  if( MaxValue <1 )
    stop("mps.sparseInterp: MaxValue <1")
  if( length(ndim)!= 1)
    stop("mps.sparseInterp: length(ndim)!= 1")
  if( length(MaxValue) != 1 )
    stop("mps.sparseInterp: length(MaxValue) != 1")
  if( length(dim(timage)) != ndim )
    stop("mps.sparseInterp: length(dim(timage)) != ndim")
  if( length(dim(iimage)) != ndim )
    stop("mps.sparseInterp: length(dim(timage)) != ndim")
  if( length(dim(OutLoc)) != 2 )
    stop("mps.sparseInterp: length(dim(OutLoc)) != 2")
  if(any(unlist(dimsInLocs)<1) ) 
    stop("mps.sparseInterp: any(unlist(InLocs)<1)")
  nval <- ncol(OutLoc)
  erg  <- .C(mpsCsparseInterp,
             ndim     = as.integer(ndim),
             MaxValue = as.integer(MaxValue),
             dimTimage= as.integer(dim(timage)),
             Timage   = as.integer(timage),
             dimIimage= as.integer(dim(iimage)),
             Iimage   = as.integer(iimage),
             dimOutLoc= as.integer(dim(OutLoc)),
             OutLoc   = as.integer(OutLoc-1),
             dimsInLoc= as.integer(dimsInLocs),
             InLoc    = as.integer(unlist(InLocs)-1),
             minTotal = as.numeric(minTotal),
             acceptWrong = as.integer(acceptWrongs),
             weights   =as.numeric(weights),
             runif    = as.numeric(runif(nval)),
             dimreport= as.integer(c(MaxValue,nval)),
             report   = numeric(MaxValue*nval),
             used     = integer(nval)
             )
  list( image=mps.image(erg$Iimage,dim=dim(iimage)),
        report=t(structure(erg$report,dim=c(MaxValue,ncol(OutLoc)))),
        used  =erg$used )

}

mps.gridInterp <- function(iimage,OutLoc,star,tab,minTotal=15,weights=NULL) {
  if(! is.null(weights) )
    tab <- mps.reweight(tab,weights)
  maxv <- dim(tab)[1]
  nval <- nrow(OutLoc)
  star <- t(star)
  OutLoc <- t(OutLoc)
  if( length(dim(star))!= 2 )
    stop("error the dimension of the interpolation star is incorrect.")
  dimReport <- c(maxv,nval)
  erg <- .C(mpsCgridInterp,
            ndim     =as.integer(length(dim(iimage))),
            maxv     =as.integer(maxv),
            dimIimage=as.integer(dim(iimage)),
            Iimage   =as.integer(iimage),
            dimStar  =as.integer(dim(star)),
            star     =as.integer(star),
            dimOutLoc=as.integer(dim(OutLoc)),
            OutLoc   =as.integer(OutLoc-1),
            lenDimCross=as.integer(length(dim(tab))),
            dimCross = as.integer(dim(tab)),
            Cross    = as.numeric(tab),
            minTotal = as.double(minTotal),
            runif    = as.numeric(runif(nval)),
            dimReport= as.integer(dimReport),
            Report   = numeric(prod(dimReport))
            )
  list(image = mps.image(erg$Iimage,dim=dim(iimage)),
       report=t(structure(erg$Report,dim=dimReport))
       )
}



mps.distdataloc <- function(image,useGrid=FALSE) {
  if( length(useGrid)!=1 )
    stop("useGrid must be a single logical")
  mps.image(
  .C(mpsCdataloc,
     ndim=length(dim(image)),
      dim=as.integer(dim(image)),
     image=as.integer(ifelse(image<0,sum(dim(image)),0)),
     useGrid=as.logical(useGrid)
     )$image,dim(image))
}

mps.makeplan <- function(image,dmax=32) {
  mps.image(
  .C(mpsCmakeplan,
     ndim=length(dim(image)),
      dim=as.integer(dim(image)),
     dmax=as.integer(dmax),
     image=as.integer(ifelse(image<0,sum(dim(image)),0))
     )$image,dim(image))
}


mps.idx2pos <- function(image,idx) {
  ndim   <-   length(dim(image))
  dimerg <- c(length(dim(image)),length(idx))
  if( any(idx<1) )
    stop("illegal index")
  t(structure(.C(mpsCimagePos,
     ndim=as.integer(ndim),
     dim =as.integer(dim(image)),
     idx =as.integer(idx-1),
     dimerg=dimerg,
     erg=integer(prod(dimerg)))$erg+1,dim=dimerg))
}

mps.pos2idx  <- function(image,pos) {
  if( any(pos<1) )
    stop("illegal position")
  
  c((pos-1) %*% c(1,cumprod(dim(image))[-length(dim(image))]))+1
}


bitstring <- function(x,d) {
  sapply(0:(d-1),function(i) {((x %/% (3^i)) %% 3) -1})
}




mps.fillplans <- function(image,stepsize) {
  ii <- 1:(3^length(dim(image))-1)
  lapply(ii,function(i) {
    grid <- mps.grid(image,stepsize*2,bitstring(i,16))
    list(grid=grid,
         pgrid=mps.idx2pos(image,grid),
         locs=(bitstring(ii,16)-bitstring(i,16))*stepsize)
  })
}


mps.fillplans2D<- function(image,stepsize) {
  grid00 <- mps.grid(image,stepsize*2,c(0,0)*stepsize)
  grid00 <- grid00[image[ mps.pos2idx(image,grid00) ]<0,]
  plan00 <- matrix(c(
                    -2,-2,
                    -2,0,
                    -2,2,
                    0,-2,
                    0,2,
                    2,-2,
                    2,0,
                    2,2
                    ),byrow=TRUE,ncol=2)*stepsize

  grid11 <- mps.grid(image,stepsize*2,c(1,1)*stepsize)
  grid11 <- grid11[image[ mps.pos2idx(image,grid11) ]<0,]
  plan11 <- matrix(c(
                    -1,-1,
                    1,-1,
                    -1,1,
                    1,1,
                    -2,0,
                    0,-2,
                    -2,-2,
                    2,2,
                    -1,3,
                    1,3,
                    3,1,
                    3,-1
                    ),byrow=TRUE,ncol=2)*stepsize
  grid10 <- mps.grid(image,stepsize*2,c(1,0)*stepsize)
  grid10 <- grid10[image[ mps.pos2idx(image,grid10) ]<0,]
  plan10 <- matrix(c(
                    -1,0,
                    0,-1,
                    1,0,
                    0,1,
                    -2,-1,
                    -2,1,
                    -1,-2,
                     1,-2,
                     2,-1,
                     2,1,
                     1,2,
                     -1,2
                    ),byrow=TRUE,ncol=2)*stepsize

  grid01 <- mps.grid(image,stepsize*2,c(0,1)*stepsize)
  grid01 <- grid01[image[ mps.pos2idx(image,grid01) ]<0,]
  plan01 <- matrix(c(
                     -1,-1,
                     -1,0,
                     -1,1,
                     0,-1,
                     0,1,
                     1,-1,
                     1,0,
                     1,1
                    ),byrow=TRUE,ncol=2)*stepsize
  plan <- function(grid,plan) {
    if( length(grid)>0 )
      list(grid=grid,pgrid=mps.pos2idx(image,grid),locs=plan)
    else
      NULL
    }
  erg <- list(plan(grid00,plan00),plan(grid11,plan11),plan(grid10,plan10),plan(grid01,plan01))
  if( is.null(erg[[1]]) )
    erg <- erg[-1]
  erg
}

mps.isorot <- function(k=7) seq(0,360,length.out=k+1)[-(k+1)]

mps.whole <- function(timage,iimage,maxv=max(timage)+1,
                      rot=mps.isorot(7),
                      scale=1:4,outer=TRUE,plan=mps.fillplans2D,
                      weights=NULL,pweights=NULL,hmax=Inf,nmax=8,minTotal=1,
                      dmax=min(dim(timage))/(6*max(scale)),replica=0,debug=0,
                      alttimages=NULL,ctrlimage=NULL,
                      specificy=0,indep=0,tabmin=0){
  if( !is.null(ctrlimage) )
    iimage[iimage!=-1] <- ctrlimage[iimage!=-1]
  datapositions <- mps.findPositions(iimage)
  planimage     <- mps.makeplan(iimage,dmax)
  if(debug >1) plot(planimage)
  II <- rep(list(iimage),replica)
  steps         <- max(abs(planimage))
  report <-  matrix(0,nrow=length(iimage),ncol=maxv)
  altreport <- lapply(alttimages,function(x) matrix(0,nrow=length(iimage),ncol=maxv))
  used   <-  rep(-1,length(iimage))
  if( any(planimage==0 & iimage==-1 ) )
    stop("Missing interplotion in data ");
  for(i in steps:1) {
    # Fill Data influenced positions
    cat("step=",i,"\n")
    inf <- (1:length(iimage))[planimage==-i]
    if( length(inf) > 1 ) {
      infLoc <- mps.idx2pos(iimage,inf)
      infinLoc<- mps.findNeighbours(infLoc,datapositions,
                                    hmax=hmax,nmax=nmax,givePos=TRUE)
      if( i < steps ) {
        infinLoc2<- mps.neargrid(iimage,i,inf)
        infinLoc <- mapply(function(x,y) rbind(x,y),infinLoc,infinLoc2)
      }
      if( debug > 2 ) points(infLoc,col="blue")
      erg <- mps.sparseInterp(timage=timage,iimage=iimage,
                              OutLoc=infLoc,InLoc=infinLoc,
                              MaxValue=maxv,minTotal=minTotal,
                              acceptWrongs=0,
                              weights=weights)
      iimage <- erg$image
      if( !is.null(ctrlimage) )
        iimage[iimage!=-1] <- ctrlimage[iimage!=-1]
      if( debug > 0 ) plot(iimage)
      II <- lapply(II,function(iimage) 
        mps.sparseInterp(timage=timage,iimage=iimage,
                              OutLoc=infLoc,InLoc=infinLoc,
                              MaxValue=maxv,minTotal=minTotal,
                              acceptWrongs=0,
                              weights=weights)$image
      )
      report[inf,]<-erg$report
      altreport <- mapply(function(timage,report) {
        report[inf,]<- mps.sparseInterp(timage=timage,iimage=iimage,
                         OutLoc=infLoc,InLoc=infinLoc,
                         MaxValue=maxv,minTotal=minTotal,
                         acceptWrongs=0,
                         weights=weights)$report
        report
      }
                          ,alttimages,altreport,SIMPLIFY=FALSE)
      used[inf] <- erg$used
    }
    if( any(planimage==-i & iimage==-1 ) )
      stop("Missing interpolation in sparse ",i);
    mps <-plan(iimage,2^(i-1))
    if(debug > 2) for(k in 1:length(mps) )
      if( !is.null(mps[[k]]) )
        points(mps[[k]]$grid,col=k)
    for( P in mps ) {
      if( is.null(P) ) next
                                        #cat("[")
      if( any(planimage[P$pgrid]!=i) | any(iimage[P$pgrid]!=-1) )
        stop("Interpolating given points")
      OutLoc <- mps.idx2pos(iimage,P$pgrid)
      if( debug> 2 ) points(OutLoc,col="red")
      star <- rbind(0,P$locs)
      tab <- mps.xtabulate(timage,star,
                           scale,rot,outer,1,weights=pweights)
      if( !is.null(weights) )
        tab <- mps.reweight(tab,weights)
      if( specificy!=0 || indep!=0 || tabmin!=0 )
        tab <- mps.changespecificy(tab,specificy,indep,tabmin)
      erg <- mps.gridInterp(iimage,OutLoc,
                            star,
                            tab,minTotal,weights)
      iimage <- erg$image
      if( !is.null(ctrlimage) )
        iimage[iimage!=-1] <- ctrlimage[iimage!=-1]
      
      if( debug >4 ) print(erg$report)
      report[P$pgrid,]<-erg$report
      altreport <- mapply(function(timage,report) {
        tab <- mps.xtabulate(timage,star,
                             scale,rot,outer,1)
        if( !is.null(weights) )
          tab <- mps.reweight(tab,weights)
        if( specificy!=0 || indep!=0 || tabmin!=0 )
          tab <- mps.changespecificy(tab,specificy,indep,tabmin)
        report[P$pgrid,]<-mps.gridInterp(iimage,OutLoc,
                            star,
                            tab,minTotal,weights)$report
        report
      },alttimages,altreport,SIMPLIFY=FALSE)
      if( debug > 0 ) plot(iimage)
      II <- lapply(II,function(iimage)
                   mps.gridInterp(iimage,OutLoc,
                                  star,
                                  tab,minTotal,weights)$image
                   )
      if( any( iimage[P$pgrid]==-1 )){
        points(OutLoc,col="red")
        stop("Point not interpolated")
      }
      #cat("]")
    }
    if( any(planimage==i & iimage==-1 ) )
      stop("Missing interpolation in grid ",i);
  }
  xreport <- report/c(report %*% rep(1,ncol(report)))
  entropy <- -(log(xreport)*xreport) %*% rep(1,ncol(report))
  varll <- (log(xreport)^2*xreport) %*% rep(1,ncol(report)) -entropy^2
  if( ! is.null(ctrlimage) ) {
    likelihood <- xreport[(c(ctrlimage)*nrow(xreport))+1:nrow(xreport)]
    likelihood <- mps.floatimage(likelihood,dim(iimage))
  } else {
    likelihood<-NULL
  }
  invisible(list(image=iimage,report=report,replica=II,altreport=altreport,
                 likelihood=likelihood,
                 entropy=mps.floatimage(entropy,dim(iimage)),
                 varll=mps.floatimage(varll,dim(iimage))))
}



mps.findPositions <- function(image,nacode=-1,...) {
  present <- (1:length(image))[image != nacode]
  dd <- cumprod(dim(image))
  t(t(outer(present-1,dd,"%%")) %/% c(1,dd[-length(dd)])) +1
}


mps.neargrid <-function(iimage,level,idx) {
  dimerg <- as.integer(c(2^length(dim(iimage)),length(idx)))
  erg <- structure(.C(mpsCneargrid,
     ndim=length(dim(iimage)),
     dim =dim(iimage),
     level=as.integer(level),
     nidx=length(idx),
     idx=as.integer(idx),
     erg=integer(prod(dimerg)))$erg,dim=dimerg)
  lapply(1:ncol(erg),function(i) {e<-erg[,i];mps.idx2pos(iimage,e[e!=-1]+1)})
}
  

mps.findNeighbours <- function(x,y=mps.findPositions(image,nacode),
                               hmax,nmax=nrow(y),nmin=5,image,
                               nacode=-1,givePos=FALSE) {
  n <- nrow(x)
  m <- nrow(y)
  h <- sqrt(c((x[rep(1:n,m),]-y[rep(1:m,each=n),])^2 %*% rep(1,ncol(x))))
  dim(h) <- c(n,m)
  erg <- lapply(1:n,function(i) {
    h <- h[i,]
    i <- (1:length(h))[h<hmax];
    if( length(i) < nmin )
      i <-  order(h)[1:nmin]
    else {
      i <- i[order(h[i])]
      if( length(i) > nmax )
        i <- i[1:nmax]
    }
    i
  })
  if(givePos)
    lapply(erg,function(i) y[i,,drop=FALSE])
  else
    erg
}

mps.ExampleOps <- function(d,n,...,p=0.5) {
  X <- rep(-1,prod(d))
  X[sample(1:length(X),n)] <- runif(n)<p
  mps.image(X,d,...)
}


mps.grid <- function(image,step,shift=rep(0,length(dim(image)))) {
  d <- dim(image)
  dd <- lapply(1:length(dim(image)), function(j) {
    i <- dim(image)[j]
    x <- seq(1+shift[j]%%step,i,by=step)
    x <- x[x>0]
    #x+as.integer(floor((i-x[length(x)])/2))
  })
  X <- matrix(dd[[1]])
  if( length(d) > 1 )
    for( i in 2:length(d) ) 
      X <- cbind(X[rep(1:nrow(X),length(dd[[i]])),],
                 rep(dd[[i]],each=nrow(X)))
  X
}


mps.reportimage <- function(erg) {
  report <- erg$report
  mps.floatimage(report[,2]/(report %*% rep(1,ncol(report))),dim(erg$image))
}
