\name{mps.gridInterp}
\alias{mps.gridInterp}
\title{MPS interpolation from grided locations}
\description{
  This routine does an MPS-Interpolation for typical points on a grid
  based on a prior tabulation from a training image.
}
\usage{
mps.gridInterp(iimage,OutLoc,star,tab,minTotal=15,weights=NULL) 
}
\arguments{
  \item{iimage}{The image to be interpolated}
  \item{OutLoc}{The locations to be interpolated as a data matrix}
  \item{star}{The relative locations to be used for interpolation as a
    data matrix. }
  \item{tab}{The crosstabulation of the interpolation point and the star
    from the training image.}
  \item{minTotal}{The minimum summed weight to be used for
    interpolation.}
  \item{weights}{Weights to rescale probabilities.}
}
\value{
  A list with two elements:
  \item{image}{The updated iimage}
  \item{report}{A matrix with rows giving unscaled conditional
    distributions from the training image for all OutLoc
  }
}
\details{
  If mintotal is not reached we reduce the number of conditioning locations.
}
\references{
  
}
\seealso{
}
\examples{
data(chanalEx1)
bwTraining <- mps.cutoff(chanalEx1)
exampleOps <- mps.ExampleOps(d=c(64,64),16)
plot(exampleOps)
positions <- mps.findPositions(exampleOps)
points(positions,col="red")
mygrid8 <- mps.grid(exampleOps,8)
mydata8 <- mps.findNeighbours(mygrid8,positions,hmax=3,nmax=4,nmin=3,givePos=TRUE)
points(mygrid8,col="green")
sp8 <- mps.sparseInterp(bwTraining,exampleOps,mygrid8,mydata8,2)
plot(sp8$image)
points(positions,col="red")
points(mygrid8,col="green")

mygrid4 <- mps.grid(exampleOps,4)
points(mygrid8,col="blue")
points(mygrid4,col="green")

star4 <- mps.locSetStar(4,4,0)
star4
ct4 <-  mps.tabulate(bwTraining,star4)

sp4 <- mps.gridInterp(sp8$image,mygrid4,star4,ct4)
plot(sp4$image)
points(mygrid4,col="blue")


sp4$report




}
\author{K.Gerald van den Boogaart}
\keyword{}