\name{mps.sparseInterp}
\alias{mps.sparseInterp}
\alias{mps.reportimage}
\title{MPS interpolation from arbitrary locations}
\description{
  This routine does an MPS-interpolation for atypical points without
  prior tabulation. 
}
\usage{
mps.sparseInterp(timage,iimage,OutLoc,InLocs,
                             MaxValue=max(c(timage))+1,
                             minTotal=15,
                             acceptWrongs=0,
                             weights=rep(1,MaxValue))
mps.reportimage(erg)
}
\arguments{
  \item{timage}{The training image as an mps.image.}
  \item{iimage}{The interpolation image as an mps.image.}
  \item{OutLoc}{a data matrix giving the locations to interpolate}
  \item{InLocs}{a list of matrices giving the locations to be used for
    each Interpolation point.
  }
  \item{MaxValue}{The number of pixelvalues to be used max()+1}
  \item{minTotal}{The minimum number of locations in the training image
    to be used to estimate a conditional distributions. If this number
    cannot, be reached the number of conditioning points is reduced.
  }
  \item{acceptWrongs}{The number of errors to accept. This special
    feature only exists with the sparse Interpolator and is to be
    considered as exeperimentally.}
  \item{weights}{}
  \item{erg}{A result of \code{mps.sparseInterp}}
}
\value{
  a list of three elements
  \item{image}{An image like Iimage, but with interpolated locations}
  \item{report}{A matrix, the rows contain the unscaled conditional
    distribution for all the interpolated points.
  }
  \item{used}{a numeric vector giving the number of conditioning
    locations actually used}

  \code{mps.reportimage} return an image of the (scaled) conditional
  probability of the second category during simulation.
  
}
\details{
}
\references{
}
\seealso{
}
\examples{
data(chanalEx1)
imc <- mps.cutoff(chanalEx1)
exampleOps <- mps.ExampleOps(d=c(20,20),15)
plot(exampleOps)
positions <- mps.findPositions(exampleOps)
points(positions,col="red")
mygrid <- mps.grid(exampleOps,5)
mydata <- mps.findNeighbours(mygrid,positions,hmax=3,nmax=4,nmin=3,givePos=TRUE)
points(mygrid,col="green")
spI <- mps.sparseInterp(imc,exampleOps,mygrid,mydata,2)
plot(spI$image)
points(positions,col="red")
points(mygrid,col="green")


}
\note{rotation, scaling and symmetry are not yet supported with sparse
  Interpolation
}
\author{K.Gerald v.d. Boogaart}
\keyword{}