\name{mps.whole}
\alias{mps.whole}
\title{Executes the whole MPS-algorithm.}
\description{
  Does conditional simulation by executing the whole mps Algorithm.
}
\usage{
mps.whole(timage,iimage,maxv=max(timage)+1,
                      rot=mps.isorot(7),
                      scale=1:4,outer=TRUE,plan=mps.fillplans2D,
                      weights=NULL,pweights=NULL,hmax=Inf,nmax=8,minTotal=1,
                      dmax=min(dim(timage))/(6*max(scale)),replica=0,debug=0,
                      alttimages=NULL,ctrlimage=NULL,
                      specificy=0,indep=0,tabmin=0)
}
\arguments{
  \item{timage}{The training image.}
  \item{iimage}{The interpolation image for which a conditional
    simulation should be created}
  \item{maxv}{The maximum value of the indexed images}
  \item{rot}{A vector of rotations in which the training image should be
    used.}
  \item{scale}{A vector of scales in which the training image should be
    used.}
  \item{outer}{Logical. If TRUE each scale is used with each rotation.}
  \item{plan}{An algorithm to determine the simulation sequence}
  \item{weights}{The weights to be used for the different facies in
    \code{mps.sparseInterp}. }
  \item{pweights}{The weights to be used for the different facies in
    \code{mps.xtabulate}. }
  \item{hmax}{The \code{hmax} to be used in \code{mps.findNeighbours}.}
  \item{nmax}{The \code{nmax} to be used in \code{mps.findNeighbours}.}
  \item{minTotal}{The \code{minTotal} to be used in
    \code{mps.sparseInterp}.}
  \item{dmax}{The maximum grid spacing to be used.}
  \item{replica}{The number of additional replica of the process to be
    generated. }
  \item{debug}{A debug level. The higher the debug level is set, the
    more output documenting the path of execution is generated.}
  \item{alttimages}{Alternative timages for which the conditional
    distributions are also computed and reported. }
  \item{ctrlimage}{If given the likelihood of this image is computed
    during the conditional simulation.}
  \item{specificy}{A parameter to controll the specificy of the
    conditional distributions. 0 means unmodified conditional
    distributions. positive values decrease entropy.}
  \item{indep}{The portion of independent noise to be added.}
  \item{tabmin}{The minimum probability value should not be further
    downscaled. A more precise definition can be found in the code given
    under example.}  }
\value{
  A list containing
  \item{image}{The simulated image}
  \item{report}{A report as in \link{mps.sparseInterp}}
  \item{replica}{A list of replica simulations of image}
  \item{altreport}{List of reports for the alternate training images}
  \item{likelihood}{The likelihood to observe the controll image.}
  \item{entropy}{a floatimage with the entropies of the the conditional
    distributions. }
  \item{varll}{variance of log-likelihoods???}
  
}
\details{
  The algorithm uses \code{mps.makeplan} based on
  \code{mps.findPositions} to generate the sequence of big steps of
  computation. In each step it first uses \code{mps.neargrid} and
  \code{mps.sparseInterp} to interpolate points with given near data
  locations. Than \code{plan} is used to generate a computation sequence
  for the remaining grid positions. The corresponding conditiona
  distributions than crosstabulated by \code{mps.xtabluate} and
  eventually modified
  according to weights with mps.reweight and changed in specificy using
  on \code{mps.changespecifiy}. The acutall interpolation is then done
  by \code{mps.gridInterp} based on the tabulation.
}
\references{
}
\seealso{
}
\examples{
mps.changespecificy <- function(tab,specificy=0,indep=0,min=0,
                                mar=apply(tab,1,sum)) {
  osum <- sum(tab)
  tab <- tab/sum(tab)
  mar <- mar/sum(mar)
  R<-mar
  for( i in 2:length(dim(tab)))
    R <- R %o% mar
  tab <-(exp(log(((1-indep)*tab/R+indep+min))*exp(specificy))-min)*R
  as.table(tab/sum(tab)*osum)
}


observed <- mps.ExampleOps(d=c(20,20),10)
mps.grid(observed,c(3,17,210))
}
\author{K.Gerald van den Boogaart}
\keyword{}