/*

C-Programms for Package mps

(C) 2006 by K.Gerald van den Boogaart, Greifswald

This package is under the Gnu Public License

Public .C Routines:

mpsCtabulate:  tabulates the joint distribution around a location in an
image. 

mpsCsparseInterp: 


Data Types:
MyImage : A container of image data 


Static Internal Routines:

imageGet0: Get the value at the current position of the index in the image
imageGet : Get the value at a position of the image 
imageIdx : Get storage location of a position of the image
imageSpecialDelta: Get value near the current location
imageStart: Start the iteration over all pixels of an image
imageStep:  Steps over all pixels of an image
*imageSize : Computes the size of an image


Change Log:


*/

#include <R.h>
#include <Rinternals.h>
#include <R_ext/Rdynload.h>
#include <R_ext/PrtUtil.h>

#define MaxDim 5   /* Maximum Number of spatial dimensions */
#define MAXVAL 255 /* Maximum Number of different pixel values */

/*
gcc -I/usr/local/lib/R/include -I/usr/local/lib/R/include  -I/usr/local/include  -Wall -pedantic  -fpic  -g -O2 -std=gnu99 -c imagemps.c -o imagemps.o

*/

/** MyImage

Internal Strukture for image processesing 


*/


typedef struct  {
    const int ndim;   /* Number of dimensions */
    const int *dim;   /* Size of dimensions */
    const int *data;  /* The actual data */
    int   pos; /* Position of the current index */
    int   i[MaxDim];     /* An index to be interated */
}  MyImage ;

typedef struct  {
    const int ndim;   /* Number of dimensions */
    const int *dim;   /* Size of dimensions */
    double *data;  /* The actual data */
    int   pos; /* Position of the current index */
    int   i[MaxDim];     /* An index to be interated */
}  MyFImage ;



/* xnode 
   xnode is the central structure of the storing algorithm 


typedef struct Xnode {
    int    value;
    double weight;
    int    N;
    struct Xnode  * nextValues;
} xnode;
*/

/* imageGet0 
   I : The image with a current position


*/

static int imageGet0(const MyImage *I) {
    return( I->data[I->pos] );
}

/* imageGet(MyImage *I,int *i) 
MyImage I  : The image to access
int i[]    : The index
This is a helper function to gain access to an Image of arbitrary dimension
*/

static int imageGet(const MyImage *I,const int i[]) {
    int ndx=0;
    for(int k=I->ndim-1; k>=0 ;k--) {
	int nd= I->dim[k];
	int v = i[k];
	if( v <0 || v >= nd )
	    error("Out of bound in imageGet");
	ndx = ndx*nd + v;
    }
    return( I->data[ndx] );
}

static int imageGet2(const MyImage *I,const int i[],const int j[]) {
    int ndx=0;
    for(int k=I->ndim-1; k>=0 ;k--) {
	int nd= I->dim[k];
	int v = i[k]+j[k];
	if( v <0 || v >= nd )
	    return -1;
	ndx = ndx*nd + v;
    }
    return( I->data[ndx] );
}


static int imageGetMir(const MyImage *I,const int i[]) {
    int ndx=0;
    for(int k=I->ndim-1; k>=0 ;k--) {
	int nd= I->dim[k];
	int v = i[k];
	if( v < 0 ) v = -v;
	v %= 2*nd;
	if( v>=nd ) 
	    v=2*nd-v-1;
	if( v <0 || v >= nd )
	    error("Out of bound in imageGetMir");
	ndx = ndx*nd + v;
    }
    return( I->data[ndx] );
}


/* imageGet(MyImage *I,int *i) 
MyImage I  : The image to access
int i[]    : The index
This is a helper function to gain storeage positions in an Image 
of arbitrary dimension
*/

static int imageIdx(const MyImage *I,const int i[]) {
    int ndx=0;
    for(int k=I->ndim-1 ; k>=0 ;k--) {
	int nd= I->dim[k];
	int v = i[k];
 	if( v <0 || v >= nd )
	    error("Out of bound in imageIdx");
	ndx = ndx*nd + v;
    }
    return( ndx );
}


static int imageIdx2(const MyImage *I,const int i[]) {
    int ndx=0;
    for(int k=I->ndim-1 ; k>=0 ;k--) {
	int nd= I->dim[k];
	int v = i[k];
 	if( v <0 || v >= nd )
	    return(-1);
	ndx = ndx*nd + v;
    }
    return( ndx );
}

static void imagePos(const MyImage *I,int j,int i[]) {
    int ndx=0;
    for(int k=0 ; k< I->ndim ;k++) {
	i[k] = j % I->dim[k];
	j    = j/I->dim[k];
    }
}

static double imageDist(const MyImage *I,int i,int j) {
    int iidx[MaxDim];
    int jidx[MaxDim];
    imagePos(I,i,iidx);
    imagePos(I,j,jidx);
    double sum=0.0,tmp;
    for(int k=0;k<I->ndim;k++)
	tmp=(double) (iidx[k]-jidx[k]),sum+=tmp*tmp;
    return(sqrt(tmp));
}

static int imageGridPos(const MyImage *I,int j,int over) {
    int idx[MaxDim];
    int k=0;
    int px;
    int i;
    for(int i=0;i<I->ndim;i++)
	k |= I->dim[i];
    for(px=0;px<16;px++)
	if( (k>>px) <= 0 )
	    break;
    imagePos(I,j,idx);
    k=0;
    for(int i=0;i<I->ndim;i++)
	k |= idx[i];
    for(i=0;i<px-over;i++)
	if( k>>i & 1 )
	    return(i);
    return(i); 
}



static int imageSpecialDelta(const int *p1, const int *p2, const MyImage*T) {
    int ndx=0;
    for(int k=T->ndim-1 ; k>=0 ; k--) {
	int nd= T->dim[k];
	int v=T->i[k]+p2[k]-p1[k];
	if( v <0 || v >= nd )
	    return -1;
	ndx = ndx*nd + v;
    }
    return( T->data[ndx] );
} 



/* imageStart/imageStep 
   Iterates of the image by
   for(imageStart(&im);imageStep(&im);) {
      imageGet(&im);
      im->pos;
      im->i; 
   }

*/

static void imageStart(MyImage *I) {
    for(int i=0;i<I->ndim;i++)
	I->i[i] = 0;
    I->i[0]=-1;
    I->pos=-1;
}

static int imageStep(MyImage *I) {
    int i;
    I->pos++;
    for(i=0;i<I->ndim;i++) {
	if( ++(I->i[i]) < I->dim[i] )
	    break;
	I->i[i] = 0;
    }
    return( i < I->ndim );
}

/*
  sizeImage
  Computes the size of data in an image.
  MyImage *I ; 

*/
static int imageSize(MyImage *I) {
    int p=1;
    for(int i=0; i< I->ndim ; i++ ) 
	p *= I->dim[i];
    return(p);
    }


/*
static int imageShiftIdx(MyImage *I,int pos,int *delta) {
    int pd = 0;
    int nf = 1;
    for(int i=I->ndim-1; i>=0 ; i--) 
	pd = pd * I->dim[i] + delta[i];
    for(int i=0;i<I->ndim;i++) {
	int k = (pos/nk) % I->dim[i];
	if( k+delta <0 || k+delta >= I->dim[i] )
	    return -1;
	nk *= I->dim[i];
    } 
    return(pd);
	
}
*/
/** 
mpsCtabulate tabulates the joint distribution around a location in an
image. 

Arguments:
int *maxValue; The number of different values in the image.
int *ndim; The number of dimensions of the image.
int *dim ; The dimensions of the image;
int *image; The integer(prod(dim)) array of image data
int *ndif; The number of relative location vectors
int *minI; The lower limits of the scanning region
int *maxI; The upper limits of the scanning region
int *difdim; The dimension (2)  of the dif array
int *dif;  The set of location differences to be scanned
int *erg;  a table of size maxI^dimdim[0], must be initialized    
*/




extern void mpsCtabulate(const int *maxValue,
			 const int *ndim,
			 const int *dim, 
			 const int *image,
			 const int *minI,
			 const int *maxI,
			 const int *difdim,
			 const int *dif,
			 int *erg
    ) {
    int MaxV = *maxValue; /* Values x in the image are 0<=x<MaxV */
    int Ndim = *ndim;     /* Number of dimensions of the image */
    int Kmax = difdim[0]; /* Number of lookup points */
    int loc[MaxDim]; /* location coordinates processed */ 
    int fac[MaxDim]; /* Factors for the location coordinates */
    int *delta=(int*)R_alloc(Kmax,sizeof(int)); /* index difference for lookup points */
    int *kfac =(int*)R_alloc(Kmax,sizeof(int)); /* factors for addressing in erg */
    int u;    /* An index into erg */
    int	ipos; /* An index into image */
    int k;    /* The offset number */
    int d;    /* The dimension processed */
    int DeltaMin,DeltaMax; /* The maximal offsets */
    int sizeImage; /* Size of the image */
    int ergsize;   /* Size of the result */
    /* Checking prerequirements */
    if( Ndim > MaxDim || Ndim < 1 ) {
	error("mpsCtabulate: only up to 1-5 dimensions supported");
	return;
    }
    if( MaxV < 2 ) {
	error("mpsCtabulate: stupid number of colors");
	return;
    }
    if( difdim[1] != Ndim )
	error("mpsCtabulate: wrong number of dimensions in dif");
    if( difdim[0] < 1 )
	error("mpsCtabulate: stupid dif");
    sizeImage=1;
    for(d=0;d< Ndim;d++)
	sizeImage*=dim[d];
    if( sizeImage <= 0 )
	error("mpsCtabulate: stupid image size");
    for(ipos=0;ipos<sizeImage;ipos++)
	if( image[ipos] < 0 || image[ipos] >= MaxV )
	    error("mpsCtabulate: illegal value in image");
    for(d=0;d< Ndim;d++) {
	if( minI[d] > maxI[d] ) {
	    error("mpsCtabulate: min and max must be ordered");
//	    tmp=minI[d];minI[d]=maxI[d];maxI[d]=tmp;
	}
	DeltaMin=DeltaMax=0;
	for(k=0;k<Kmax;k++) {
	    if( DeltaMin > dif[k+Kmax*d] )
		DeltaMin = dif[k+Kmax*d];
	    if( DeltaMax < dif[k+Kmax*d] )
		DeltaMax = dif[k+Kmax*d];
	}
	if( minI[d]+DeltaMin < 0 || maxI[d]+DeltaMax>dim[d] )
	    error("mpsCtabulate: minI,maxI out of image");
    }
    /* Preparation***************************************  */
    ergsize=1;
    for(k=0;k<Kmax;k++) {
	if( ergsize >= 0x7FFFFFFF/MaxV ) /** No MAX_INT found */
	    error("mpsCtabulate: erg to big");
	ergsize*=MaxV;
    }
    for(u=0;u<ergsize;u++)
	erg[u]=0;
    /* ** Factor for image indices*/
    ipos=1;
    for(d=0;d<Ndim;d++) {
	loc[d]=minI[d];
	fac[d]=ipos;
	ipos*=dim[d];
    }
    /* ** Lokation differences */
    u = 1;
    for(k=0;k<Kmax;k++) {
	kfac[k]=u;
	u*=MaxV;
	delta[k]=0;
	for(d=0;d<Ndim;d++) 
	    delta[k]+= fac[d]*dif[k+Kmax*d];
    }
    /* ** Working point in the image*/
    ipos=0;
    for(d=0;d<Ndim;d++) {
	ipos+= loc[d]*fac[d];
    }
    /* ipos is now location in the image*/
    loc[0]--; ipos --;
    /* Loop over the image ********************************* */
    for(;;) { 
	for(d=0;d<Ndim;d++) {
	    ipos += fac[d];
	    if( ++loc[d] < maxI[d] )
		break;
	    ipos   -= (loc[d]-minI[d])*fac[d];
	    loc[d]  = minI[d];
	}
	if( d>=Ndim )
	    break; /* Ready */
	u = 0;
	for(k=0;k<Kmax;k++) 
	    u += kfac[k]*image[ipos+delta[k]];
	erg[u]++;
    }
}


extern void mpsCwimage(const int *maxValue,
		       const int *ndim,
		       const int *dim,
		       const int *image,
		       const double * weights,
		       const double *dist,
		       double * wimage) { 
    
    MyImage im={*ndim,dim,image};
    int idx1[MaxDim];
    int idx2[MaxDim];
    int len = imageSize(&im);
    double wmean=0.0;
    double w;
    int nok=0;
    for(int i=0;i<len;i++) {
	if( image[i]>=0 && image[i]<*maxValue ) {
	    wmean+=weights[image[i]];
	    nok++;
	}
    }
    wmean/=nok;
    for(int i=0;i<len;i++) {
	w=0.0;
	for(int j=0;j<len;j++) {
	    if( image[j]>=0 && image[j]<*maxValue && 
		imageDist(&im,i,j) <= *dist ) {
		w+=weights[image[j]]-wmean;
	    }
	}
	wimage[i]=w;
    }
    
}

/** 
mpsCtabulate tabulates the joint distribution around a location in an
image. 

Arguments:
int *maxValue; The number of different values in the image.
int *ndim; The number of dimensions of the image.
int *dim ; The dimensions of the image;
int *image; The integer(prod(dim)) array of image data
int *ndif; The number of relative location vectors
int *minI; The lower limits of the scanning region
int *maxI; The upper limits of the scanning region
int *difdim; The dimension (2)  of the dif array
int *dif;  The set of location differences to be scanned
int *erg;  a table of size maxI^dimdim[0], must be initialized    
*/




extern void mpsCweightedtabulate(const int *maxValue,
				 const int *ndim,
				 const int *dim, 
				 const int *image,
				 const int *minI,
				 const int *maxI,
				 const int *difdim,
				 const int *dif,
				 const double *wimage,
				 double *erg
    ) {
    int MaxV = *maxValue; /* Values x in the image are 0<=x<MaxV */
    int Ndim = *ndim;     /* Number of dimensions of the image */
    int Kmax = difdim[0]; /* Number of lookup points */
    int loc[MaxDim]; /* location coordinates processed */ 
    int fac[MaxDim]; /* Factors for the location coordinates */
    int *delta=(int*)R_alloc(Kmax,sizeof(int)); /* index difference for lookup points */
    int *kfac =(int*)R_alloc(Kmax,sizeof(int)); /* factors for addressing in erg */
    int u;    /* An index into erg */
    int	ipos; /* An index into image */
    int k;    /* The offset number */
    int d;    /* The dimension processed */
    int DeltaMin,DeltaMax; /* The maximal offsets */
    int sizeImage; /* Size of the image */
    int ergsize;   /* Size of the result */
    /* Checking prerequirements */
    if( Ndim > MaxDim || Ndim < 1 ) {
	error("mpsCtabulate: only up to 1-5 dimensions supported");
	return;
    }
    if( MaxV < 2 ) {
	error("mpsCtabulate: stupid number of colors");
	return;
    }
    if( difdim[1] != Ndim )
	error("mpsCtabulate: wrong number of dimensions in dif");
    if( difdim[0] < 1 )
	error("mpsCtabulate: stupid dif");
    sizeImage=1;
    for(d=0;d< Ndim;d++)
	sizeImage*=dim[d];
    if( sizeImage <= 0 )
	error("mpsCtabulate: stupid image size");
    for(ipos=0;ipos<sizeImage;ipos++)
	if( image[ipos] < 0 || image[ipos] >= MaxV )
	    error("mpsCtabulate: illegal value in image");
    for(d=0;d< Ndim;d++) {
	if( minI[d] > maxI[d] ) {
	    error("mpsCtabulate: min and max must be ordered");
//	    tmp=minI[d];minI[d]=maxI[d];maxI[d]=tmp;
	}
	DeltaMin=DeltaMax=0;
	for(k=0;k<Kmax;k++) {
	    if( DeltaMin > dif[k+Kmax*d] )
		DeltaMin = dif[k+Kmax*d];
	    if( DeltaMax < dif[k+Kmax*d] )
		DeltaMax = dif[k+Kmax*d];
	}
	if( minI[d]+DeltaMin < 0 || maxI[d]+DeltaMax>dim[d] )
	    error("mpsCtabulate: minI,maxI out of image");
    }
    /* Preparation***************************************  */
    ergsize=1;
    for(k=0;k<Kmax;k++) {
	if( ergsize >= 0x7FFFFFFF/MaxV ) /** No MAX_INT found */
	    error("mpsCtabulate: erg to big");
	ergsize*=MaxV;
    }
    for(u=0;u<ergsize;u++)
	erg[u]=0;
    /* ** Factor for image indices*/
    ipos=1;
    for(d=0;d<Ndim;d++) {
	loc[d]=minI[d];
	fac[d]=ipos;
	ipos*=dim[d];
    }
    /* ** Lokation differences */
    u = 1;
    for(k=0;k<Kmax;k++) {
	kfac[k]=u;
	u*=MaxV;
	delta[k]=0;
	for(d=0;d<Ndim;d++) 
	    delta[k]+= fac[d]*dif[k+Kmax*d];
    }
    /* ** Working point in the image*/
    ipos=0;
    for(d=0;d<Ndim;d++) {
	ipos+= loc[d]*fac[d];
    }
    /* ipos is now location in the image*/
    loc[0]--; ipos --;
    /* Loop over the image ********************************* */
    for(;;) { 
	for(d=0;d<Ndim;d++) {
	    ipos += fac[d];
	    if( ++loc[d] < maxI[d] )
		break;
	    ipos   -= (loc[d]-minI[d])*fac[d];
	    loc[d]  = minI[d];
	}
	if( d>=Ndim )
	    break; /* Ready */
	u = 0;
	for(k=0;k<Kmax;k++) 
	    u += kfac[k]*image[ipos+delta[k]];
	erg[u]+=wimage[ipos];
    }
}



/* 
Routine for the initial phase of the MPS:
Getting conditional distributions from unstructured locations.
int * ndim        :  The number of spatial dimensions
int * MaxValue    :  The maximum color value accepted in the images +1
int * dimTimage   :  The dimension attribute of the Training image
int * Timage      :  The training image
int * dimIimage   :  The dimension attribute of the Interpolation image
int * dimOutLoc   :  The dimension attribute of outloc
int * OutLoc      :  The locations to be interpolated as colums
int * dimsInLoc   :  the unlisted sequence of the dimension attributes  of InLoc
int * InLoc       :  unlisted list of column datasets giving the positions of
                     neibouring locations to be used for interpolation 
double * runif    :  A numeric(nrow(OutLoc)) of [0,1] random uniforms for the 
                     interpolation
int * dimreport   :  The dimension of the report, should be c(
*/


    
extern void mpsCsparseInterp(
    const int *ndim,
    const int *MaxValue,
    const int *dimTimage,
    const int *Timage,
    const int *dimIimage,
    int *Iimage,
    const int *dimOutLoc,
    const int *OutLoc,
    const int *dimsInLoc,
    const int *InLoc,
    const double *minTotal,
    const int *acceptWrongs,
    const double *weights,
    const double *runif,
    const int *dimreport,
    double *report,
    int *used) {
    int Ndim = *ndim;          /* Number of dimension of the image */
    int MaxV = *MaxValue;      /* Number of Pixelvalues */
    int Nval = dimOutLoc[1];   /* Number of Interpolation sites */
    int wrongs=0;              /* The number of nonfitting values */
    int useLocs=0;             /* The number of locations to be actually used 
				  This decreases, when minTotal is not reached
			       */
    int l=0;                   /* the secondary locations loop index */ 
    int v2=0;                  /* The value in the secondary locations 
				  of the training image*/
    int v=0;                   /* A value in the interpolation image */
    int v1=0;                  /* Value in the primary location of the 
				  training image */
    double total=0;               /* The total number of training information 
				  with the given combination.
			       */
    double myweight;
    MyImage timage = {Ndim,dimTimage,Timage};
    MyImage iimage = {Ndim,dimIimage,Iimage};
    /* Checking */
    if( Ndim <1 || Ndim >= MaxDim )
	error("mpsCsparseInterp: Number of spatial dimensios out of range");
    if( dimOutLoc[0]!=Ndim )
	error("mpsCsparseInterp: Dimension missfit int dimOutLoc");
    if( dimsInLoc[0] !=Ndim )
	error("mpsCsparseInterp: Dimension missfit int dimsInLoc");
    if( *MaxValue < 2 || *MaxValue > MAXVAL )
	error("mpsCsparseInterp: Nonsense MaxValue");
    if( dimreport[1] != dimOutLoc[1] || dimreport[0]!= MaxV )
	error("mpsCsparseInterp: Wrong dimension for report");
    if( Nval < 1 )
	error("mpsCsparseInterp: Nothing to be done");
    /* Preparation */ 
    /* Loop over Interpolation Locations */
    for(int k=0;k<Nval;k++) {
	/* Initialise conditional */
	if( dimsInLoc[0] != Ndim )
	    error( "mpsCsparseInterp: Some InLoc has wrong number of dimension." );
	for(int i=0;i<MaxV;i++)
	    report[i]=0.0;
	/* Loop over training image */
	/*Rprintf(".",k);*/
	if( dimsInLoc[1] < 1 )
	    error("mpsCsparseInterp: No points given");
	for(useLocs = dimsInLoc[1]; useLocs > 0; useLocs--) {
	    for(imageStart(&timage);imageStep(&timage);) {
		/* Loop over lookup locations */
		myweight=1;
		wrongs=0;
		for(l=0;l<useLocs;l++) {
		    const int * otherLoc = InLoc+dimsInLoc[0]*l;
		    v2=imageSpecialDelta(OutLoc,
					 otherLoc,
					 &timage);
		    if( v2 < 0 )
			continue;
		    myweight*=weights[v2];
		    v =  imageGet(&iimage, otherLoc);
		    if( v < 0 )
			continue;
		    if( v2 != v ) {
			wrongs++;
			if( wrongs > *acceptWrongs )
			    break;  /* l */
		    }
		}
		if( l < useLocs ) {
		    continue; /* timage */
		} 
		v1 = imageGet0(&timage);
		if( v1>=0 && v1 <MaxV )
		    report[v1]+=myweight;
	    }
	    /* compute distribution */
	    total =0.0;
	    for(v1=0;v1<MaxV;v1++)
		total += (report[v1]*=weights[v1]);
	    if( total >= *minTotal ) {
		/* Simulation */
		if( *runif==1.0 )
		    total--;
		else 
		    total = *runif * total;
		for(v1=0;v1<MaxV;v1++) {
		    total-=report[v1];
		    if( total < 0 || v1 == MaxV-1) { 
			Iimage[imageIdx(&iimage,OutLoc)] = v1;
			break;
		    }
		}
		/* Advancing to the next interpolation Location */
		*used =useLocs;
		useLocs=0;
	    } else {/*Rprintf("!");*/}
	}
	OutLoc += Ndim;
	InLoc  += dimsInLoc[0]*dimsInLoc[1];
	dimsInLoc += 2;
	report += MaxV;
	used++;
	runif++;
    }
}


/** 
    mpsCreweight
    reweights the tables
*/         

extern void mpsCreweight(const int *ndim,
		    int *dim,
		    double *tab,
		    double *weights) {
    int Ndim=*ndim;
    int MaxV=*dim;
    for(int i=0;i<Ndim;i++)
	if( dim[i]!=MaxV )
	    error("mpsCreweight: dimension does not match");
    int len=1;
    for(int i=0;i<Ndim;i++)
	len*=MaxV;
    int inc=1;
    for(int i=0;i<Ndim;i++) {
	for(int j=0;j<MaxV;j++)
	    for(int w=0;w<len;w+=inc*MaxV)
		for(int u=0;u<inc;u++)
		    tab[u+w+inc*j]*=weights[j];
	inc*=MaxV;
    }
}


/**

mpsCgridInterp interpolats the interpolation image on a grid defined by shiftgrid and stepgrid using the relativly given locations in 

*/


extern void mpsCgridInterp(
    const int * ndim,       /* Number of Dimension of the Image */
    const int * maxv,       /* The number of values to considered */
    const int * dimIimage,  /* Dimensions of the Interpolationimage */
    int * Iimage,           /* The Interpolationsimage */
    const int * dimStar,    /* Dimension of the Interpolationstart */
    const int * star,       /* The Interpolationsstar */
    const int *dimOutLoc,   /* dim of the Interpolation locations */
    const int *OutLoc,      /* The locations to be interpolated  */
    const int * lenDimCross,
    const int * dimCross, /* The dimension of weights */
    const double *Cross,   /* The crosstabulation */
    const double *minTotal, /* The minimum total */
    const double *runif,    /* Random Uniforms */
    const int *dimReport,   /* dimension of the report */
    double *Report          /* The conditional distributions used*/
    ) {
    /* Getting constants */
    int Ndim = *ndim;  /* The number of dimension of the image */
    int MaxV = *maxv;  /* The number of different pixel values */
    int Npre = dimStar[1];
    int Nval = dimOutLoc[1]; /* number of interpolation locations */
    MyImage iimage = {Ndim,dimIimage,Iimage};
    double total;
    int useLocs;
    /* Checking input */
    if( Ndim < 1 || Ndim > MaxDim )
	error("mpsCgridInterp: ndim < 1 || ndim > MaxDim");
    if( MaxV <2 || MaxV >MAXVAL )
	error("mpsCgridInterp: maxv <2 || maxv >255 ");
    if( dimOutLoc[0]!=Ndim )
	error("mpsCgridInterp: dimOutLoc[0]!=ndim");
    if( dimStar[0]!=Ndim )
	error("mpsCgridInterp: dimStar[0]!=ndim");
    if( dimReport[0] != MaxV )
	error("mpsCgridInterp: dimReport[0] != MaxV ");
    if( dimReport[1] != Nval )
	error("mpsCgridInterp: dimReport[1] != Nval ");
    if( *lenDimCross != Npre )
	error("mpsCgridInterp: lenDimCross != Npre ");
    for(int i=0; i< *lenDimCross; i++)
	if( dimCross[i]!=MaxV )
	    error("mpsCgridInterp: dimCross[i]!=maxv ");
    /* Preparation */
    int lenCross=1;
    for(int i=0; i< *lenDimCross; i++)
	lenCross *= MaxV;
    /*  * Subtotal engine */
    int    * fixed     = (int*) R_alloc(Npre,sizeof(int));
    int    * fak       = (int*) R_alloc(Npre,sizeof(int));
    int    * ifix      = (int*) R_alloc(Npre,sizeof(int));
    int fakCur=1;
    for(int i=0;i<Npre;i++) {
	ifix[i]=0;
	fixed[i]=0;
	fak[i]=fakCur;
	fakCur*=MaxV;
    }
    /* Loop for interpolation */
    for(int ol=0;ol<Nval;ol++) {
	for(useLocs=Npre;useLocs>=0;useLocs--) {
	    int wIdx=0;
	    int efak=MaxV; /* for internal summing */
	    int k;
	    for(k=Npre-1;k>useLocs;k--)
		fixed[k]=0;
	    for(;k>=1;k--) {
		int v=imageGet2(&iimage,OutLoc+Ndim*ol,star+Ndim*k);
		if( v < 0 )
		    fixed[k]=0;
		else {
		    fixed[k]=1;
		    if( v>=MaxV )
			error("Image exceeds maxvalue");
		    wIdx = (wIdx + v)*MaxV;
		}
	    }
	    fixed[0]=1;
	    total=0.0;
	    for(int v=0;v<MaxV;v++) 
		Report[v]=0;
	    int owIdx=wIdx;
	    do {
		for(int k=1;k<Npre;k++) {
		    if( fixed[k] )
			continue;
		    wIdx+=fak[k];
		    if( ++(ifix[k]) < MaxV )
			break;
		    wIdx-=MaxV*fak[k];
		    ifix[k]=0;
		}
		for(int v=0;v<MaxV;v++) 
		    Report[v] += Cross[wIdx+v];
	    } while( wIdx!=owIdx );
	    total=0;
	    for(int v=0;v<MaxV;v++) 
		total += Report[v];
	    if( total < *minTotal ) {
		/* Rprintf(";");*/
		continue;
	    }
	    double portion = total*runif[ol];
	    if( portion >= total )
		error("Internal error");
	    int nofound=1;
	    for(int v=0;v<MaxV;v++) {
		if( (portion -= Report[v]) <= 0.0 ) {
		    Iimage[imageIdx(&iimage,OutLoc+Ndim*ol)]=v;
		    nofound=0;
		    break;
		}
	    }
	    
	    if( nofound ) 
		error("No Value found");
	    useLocs=-1;
	}
	if( Iimage[imageIdx(&iimage,OutLoc+Ndim*ol)]==-1 ) {
	    Rprintf("mpsCgridInterp: Couldn't interpolate point %d %lf ",useLocs,total);
	    error("mpsCgridInterp: Couldn't interpolate point ");
	}
	Report+=MaxV;
    }
    
}

extern void mpsCdataloc(const int *ndim,
			    const int *dim,
			    int *image,
			    const int *useGrid
    ) {
    int Ndim=*ndim;
    if( Ndim<1 )
	error("mpsCdataloc: This does not look like an image");
    MyImage im={*ndim,dim,image};
    int *ndxi=(int*) R_alloc(*ndim,sizeof(int));
    int *ndxj=(int*) R_alloc(*ndim,sizeof(int));
    int n = imageSize(&im);
    for(int i=0;i<n;i++) {
	int gridI = imageGridPos(&im,i,0);
	if( image[i] == 0 ) {
	    imagePos(&im,i,ndxi);
	    for(int j=0;j<n;j++) {
		int gridJ = imageGridPos(&im,j,0);
		if( *useGrid && gridJ<gridI )
		    continue;
		imagePos(&im,j,ndxj);
		double tmp=0.0;
		for(int k=0;k<Ndim;k++) {
		    double r=(double)(ndxj[k]-ndxi[k]);
		    tmp+=r*r;
		}
		int newdist = (int) floor(sqrt(tmp)); 
		if( image[j] > newdist )
		    image[j]=newdist;
	    }
	}
    }
}





extern void mpsCmakeplan(const int *ndim,
			const int *dim,
			 const int *dmax,
			int *image
    ) {
    const int yes=1;
    mpsCdataloc(ndim,dim,image,&yes);
    int Ndim=*ndim;
    if( Ndim<1 )
	error("mpsCmakeloc: This does not look like an image");
    MyImage im={*ndim,dim,image};
    int *ndxi=(int*) R_alloc(*ndim,sizeof(int));
    int *ndxj=(int*) R_alloc(*ndim,sizeof(int));
    int n = imageSize(&im);
    int MaxStep = imageGridPos(&im,0,2);
    for(int i=0;i<n;i++) {
	/*Rprintf("(%d,",image[i]);*/
	if( image[i] == 0 ) 
	    image[i] = 0;
	else {
	    int gp = imageGridPos(&im,i,2);
	    while( 1<<gp > *dmax )
		gp--;
	    if( (1<<gp) >= image[i] )
		image[i]=-(gp+1);
	    else 
		image[i]=(gp+1);
	}
	/*Rprintf("%d)",image[i]);*/
    } 
}




extern void mpsCimagePos(int *ndim,int *dim,int *idx,int* dimerg,int*erg) {
    MyImage im = {*ndim,dim,NULL};
    for(int i=0;i<dimerg[1];i++) {
	imagePos(&im,idx[i],erg+dimerg[0]*i);
    }
}

extern void mpsCneargrid(int *ndim,int *dim,int *level,int *nidx,int *idx,int *erg) {
    int iii[MaxDim];
    int jjj[MaxDim];
    MyImage im = {*ndim,dim,NULL};
    int Pattern = 0xFFFFFF << *level;
    int maxK=1<< (im.ndim);
    if( *ndim >MaxDim )
	error("mpsCneargrid: Dimension exceeds maximum");
    for(int i=0;i<*nidx;i++) {
	imagePos(&im,idx[i],iii);
	for(int k=0;k<maxK;k++) { 
	    for(int j=0;j<im.ndim;j++) {
		jjj[j] = iii[j] & Pattern;
		if( 1<<j & k )
		    jjj[j] += 1<<*level;
	    }
	    *(erg++) = imageIdx2(&im,jjj);
	}
    }
}

/*
extern mpsCvgram(int *ndim,
		 int *dimImage,
		 int *Image,
		 int *center,
		 int *dimVg,
		 int *Vg,
		 int *mean) {
    

}  
*/

static R_NativePrimitiveArgType mpsCtabulate_t[9] =      {
    INTSXP, INTSXP,INTSXP, /*MaxValue, ndim,dim */
    INTSXP, INTSXP,INTSXP, /*image,minI,maxI */
    INTSXP, INTSXP,INTSXP  /*difdim,dif,erg*/
};

static R_NativePrimitiveArgType mpsCneargrid_t[6] =      {
    INTSXP, INTSXP,INTSXP, /*MaxValue, ndim,dim */
    INTSXP, INTSXP,INTSXP, /*image,minI,maxI */
};


static R_NativePrimitiveArgType mpsCwimage_t[7] =      {
    INTSXP, INTSXP,INTSXP, /*maxValue, ndim,dim */
    INTSXP, REALSXP,REALSXP, /*image,weights,dist */
    REALSXP  /*wimage*/
};


static R_NativePrimitiveArgType mpsCweightedtabulate_t[10] =      {
    INTSXP, INTSXP,INTSXP, /*MaxValue, ndim,dim */
    INTSXP, INTSXP,INTSXP, /*image,minI,maxI */
    INTSXP, INTSXP,        /*difdim,dif*/
    REALSXP,REALSXP        /*wimage,erg*/
};


static R_NativePrimitiveArgType mpsCreweight_t[4] =      {
    INTSXP, INTSXP,  /*ndim, dim*/
    REALSXP, REALSXP /*tab, weights */
};

static R_NativePrimitiveArgType mpsCdataloc_t[4] =      {
    INTSXP, INTSXP,  /*ndim, dim*/
    INTSXP, LGLSXP  /*image */
};

static R_NativePrimitiveArgType mpsCmakeplan_t[4] =      {
    INTSXP, INTSXP, INTSXP, /*ndim, dim,dmax*/
    INTSXP /*image */
};

static R_NativePrimitiveArgType mpsCsparseInterp_t[17] = {
    INTSXP, INTSXP,INTSXP, INTSXP ,/* ndim, MaxValue, dimTimage,Timage*/
    INTSXP, INTSXP,INTSXP, /* dimIimage,Iimage,dimOutLoc */
    INTSXP, INTSXP,INTSXP, /* OutLoc, dimsInLoc, InLoc */
    REALSXP, INTSXP, /* minTotal, acceptWrongs, */
    REALSXP,	   /* weights */ 
    REALSXP, INTSXP,REALSXP,/* runif,dimreport,report */
    INTSXP                 /* used */
};

static R_NativePrimitiveArgType mpsCgridInterp_t[15] =      {
    INTSXP, INTSXP,INTSXP, /* ndim,maxv,dimIimage */
    INTSXP, INTSXP,INTSXP, /* Iimage,dimstar,star */
    INTSXP, INTSXP,INTSXP, /* dimOutLoc, OutLoc, lenDimCross*/
    INTSXP, REALSXP,REALSXP,REALSXP,  /* dimCross, Cross, minTotal, runif*/
    INTSXP, REALSXP  /* dimReport, Report*/
};


static R_NativePrimitiveArgType mpsCimagePos_t[5] =      {
    INTSXP, INTSXP,INTSXP, /* ndim,dim,idx, */
    INTSXP, INTSXP /* dimerg,erg */
};



static R_CMethodDef  cMethods[] = {
    {"mpsCtabulate", (DL_FUNC) &mpsCtabulate, 9, mpsCtabulate_t },
    {"mpsCneargrid", (DL_FUNC) &mpsCneargrid, 6, mpsCneargrid_t },
    {"mpsCwimage", (DL_FUNC) &mpsCwimage, 7, mpsCwimage_t },
    {"mpsCweightedtabulate", (DL_FUNC) &mpsCweightedtabulate, 10, mpsCweightedtabulate_t },
    {"mpsCsparseInterp", (DL_FUNC) &mpsCsparseInterp, 17, mpsCsparseInterp_t },
    {"mpsCreweight", (DL_FUNC) &mpsCreweight, 4, mpsCreweight_t },
    {"mpsCgridInterp", (DL_FUNC) &mpsCgridInterp, 15, mpsCgridInterp_t },
    {"mpsCdataloc", (DL_FUNC) &mpsCdataloc, 4, mpsCdataloc_t },
    {"mpsCmakeplan", (DL_FUNC) &mpsCmakeplan, 4, mpsCmakeplan_t },
    {"mpsCimagePos", (DL_FUNC) &mpsCimagePos, 5, mpsCimagePos_t },
    {NULL, NULL, 0}
};




extern void R_init_mps(DllInfo *info) {
    R_registerRoutines(info, cMethods, NULL, NULL, NULL);
}

extern void  R_unload_mps(DllInfo *info) {
}

