\name{mps.reweight}
\alias{mps.reweight}
\alias{mps.changespecificy}
\title{Transforming of crosstabulation tables}
\description{
  Consistent joint distributions given as crosstabulations can be
  transformed to other consistent
  joint distributions by reweighting all entries with a product of
  factors depending on the individual levels only and on each in the
  same way. 
}
\usage{
mps.reweight(tab,weights)
mps.changespecificy(tab,specificy=0,indep=0,min=0,mar=apply(tab,1,sum))
}
\arguments{
  \item{tab}{A crosstabulation result from mps.tabulate or
    mps.xtabulate}
  \item{weights}{a vector of weights for each color}
  \item{specificy}{A parameter changing the specificy of the
    distribution. 0 means no change. Positive values increase mutual
    information. Negative values decrease it.
  }
  \item{indep}{The independency part of the distribution: A value
    between 0 and 1 creates an independecy part of the given
    portion. Negative values are theoretically possible to remove the
    part again. A minimum portion of the marginal to be introduced to avoid
    problems with zeros. A zero is strictly speaking an infinite
    sepecificy and thus not removed by pure change of spefificy.}
  \item{mar}{The marginal reference distribution to be used.}
  \item{min}{An alternvate way of making infinite specificy finite by
    bound it to a minimum portion}
}
\value{
  A reweighted or modified (real) table.
}
\details{
  The training image can be seen as a model structure. However just like
  for the covariance we can
  define several parameters
  \item{marginal distribution}{reweighting varies this but in a not
    directly predictable way. This is like a mean parameter for simple
    kriging, which is related to the sill parameter for multinomial
    distributions. }
  \item{independency part}{This is like a nugget parameter of
    the correlogram. }
  \item{specificy of distribution}{ This like a shape parameter of the
    correlogramm}
  \item{scale of the location configeration}{
    This is like a range parameter of the variogramm.
  }
  All these can ony be change relative to the original joint
  distribution given by training image.
  \cr
  None of the distributions need to be closed.
}
\references{
  Boogaart, K.G. (2006) Some theory for Multiple Point Statistics: Fitting,
  checking and optimally exploiting the training
  image, Proceedings of IAMG 2006, Liege, to appear
}
\seealso{
}
\examples{

data(chanalEx1)
bwTraining <- mps.cutoff(chanalEx1)
locs <- mps.locSetStar(1,4,0)
locs
ct1 <-  mps.tabulate(bwTraining,locs)

ftable(ct1)

ftable(mps.reweight(ct1,c(10,1)))

ftable(mps.changespecificy(ct1,-1))
ftable(mps.changespecificy(ct1,-10))
ftable(round(mps.changespecificy(ct1,1),digits=2))

ftable(mps.changespecificy(ct1,indep=0.5))
ftable(mps.changespecificy(ct1,indep=1))
ftable(mps.changespecificy(ct1,-10,indep=0.1))
ftable(mps.changespecificy(ct1,-10))

ftable(mps.changespecificy(ct1,-10,indep=0.1,mar=c(1,1)))


}
\author{K.Gerald van den Boogaart}
\keyword{}